from service_linker.adapter import ServiceAdapter
from service_linker.core import ServiceManager


@ServiceManager.register_service_adapter
class MongoAdapter(ServiceAdapter):
    thread_safe = True
    service_name = "mongo"

    def do_get_connection(self, *args, **params):
        import pymongo

        client = pymongo.MongoClient(**params)
        return client

    def do_close(self, client):

        client.close()



from .adapter import ServiceAdapter
from .adapters import (
    ElasticSearchAdapter,
    KafkaConsumerAdapter,
    KafkaProducerAdapter,
    MongoAdapter,
    MySQLRawAdapter,
    RedisAdapter,
    json_decode,
    json_encode,
)
from .core import ServiceManager

__all__ = [
    "ServiceAdapter",
    "ServiceManager",
    "ElasticSearchAdapter",
    "KafkaConsumerAdapter",
    "KafkaProducerAdapter",
    "MongoAdapter",
    "MySQLRawAdapter",
    "RedisAdapter",
    "json_encode",
    "json_decode",
]



import json

from service_linker.adapters.elasticsearch_adapter import ElasticSearchAdapter
from service_linker.adapters.kafka_consumer_adapter import KafkaConsumerAdapter
from service_linker.adapters.kafka_producer_adapter import KafkaProducerAdapter
from service_linker.adapters.mongo_adapter import MongoAdapter
from service_linker.adapters.mysql_raw_adapter import MySQLRawAdapter
from service_linker.adapters.redis_adapter import RedisAdapter


def json_encode(v):
    return json.dumps(v).encode("utf-8")


def json_decode(v):
    if v is not None:
        return json.loads(v.decode("utf-8"))
    return None



from service_linker.adapter import ServiceAdapter
from service_linker.core import ServiceManager


@ServiceManager.register_service_adapter
class MySQLRawAdapter(ServiceAdapter):
    thread_safe = False
    service_name = "raw_pymysql"

    def do_get_connection(self, **params):
        import pymysql

        conn = pymysql.connect(**params)
        return conn

    def do_close(self, conn):

        conn.close()



from service_linker.adapter import ServiceAdapter
from service_linker.core import ServiceManager


@ServiceManager.register_service_adapter
class KafkaConsumerAdapter(ServiceAdapter):
    thread_safe = False
    service_name = "kafka_python_consumer"

    def do_get_connection(self, *args, **params):
        from kafka import KafkaConsumer

        consumer = KafkaConsumer(*args, **params)
        return consumer

    def do_close(self, consumer):
        consumer.close()



from service_linker.adapter import ServiceAdapter
from service_linker.core import ServiceManager


@ServiceManager.register_service_adapter
class RedisAdapter(ServiceAdapter):
    thread_safe = False
    service_name = "redis"

    def do_get_connection(self, *args, **params):
        import redis

        rc = redis.Redis(**params)
        return rc

    def do_close(self, rc):

        rc.close()



from service_linker.adapter import ServiceAdapter
from service_linker.core import ServiceManager


@ServiceManager.register_service_adapter
class ElasticSearchAdapter(ServiceAdapter):
    thread_safe = True
    service_name = "elasticsearch"

    def do_get_connection(self, *args, **params):
        from elasticsearch import Elasticsearch

        es = Elasticsearch(*args, **params)
        return es

    def do_close(self, es):
        if hasattr(es, "close"):  # version >= 7.14.0
            es.close()
        else:  # version < 7.14.0
            es.transport.close()



import logging
from abc import ABC, abstractmethod

_logger = logging.getLogger(__name__)


class ServiceAdapter(ABC):
    """
    Abstract base class for creating service adapters.

    Attributes:
        thread_safe (bool): Indicates whether the connection is thread-safe. Must be overridden in subclasses.
        service_name (str): The name of the service associated with this adapter. Must be overridden in subclasses.
    """

    thread_safe = None
    service_name = None

    def __init_subclass__(cls, **kwargs):
        """
        Validates that subclasses override the required attributes `thread_safe` and `service_name`.

        Args:
            **kwargs: Additional arguments for subclass initialization.

        Raises:
            TypeError: If `thread_safe` or `service_name` is not overridden in the subclass.
        """
        super().__init_subclass__(**kwargs)
        if cls.thread_safe is None:
            raise TypeError(f"Class '{cls.__name__}' must override 'thread_safe'")
        if cls.service_name is None:
            raise TypeError(f"Class '{cls.__name__}' must override 'service_name'")

    def get_connection(self, **params):
        """
        Retrieves a connection by delegating to the `do_get_connection` method.

        Args:
            **params: Parameters needed to establish the connection.

        Returns:
            The established connection.
        """
        _logger.debug(f"Getting connection for service '{self.service_name}'")
        connection = self.do_get_connection(**params)
        _logger.debug(
            f"Successfully retrieved connection for service '{self.service_name}'"
        )
        return connection

    def close(self, connection):
        """
        Closes a connection by delegating to the `do_close` method.

        Args:
            connection: The connection to be closed.
        """
        _logger.debug(f"Closing connection for service '{self.service_name}'")
        self.do_close(connection)
        _logger.debug(
            f"Successfully closed connection for service '{self.service_name}'"
        )

    @abstractmethod
    def do_get_connection(self, *args, **params):
        """
        Abstract method to establish a connection. Must be implemented in subclasses.

        Args:
            *args: Parameters needed to establish the connection.
            **params: Parameters needed to establish the connection.

        Returns:
            The established connection.
        """
        pass

    @abstractmethod
    def do_close(self, connection):
        """
        Abstract method to close a connection. Must be implemented in subclasses.

        Args:
            connection: The connection to be closed.
        """
        pass



from service_linker.adapter import ServiceAdapter
from service_linker.core import ServiceManager


@ServiceManager.register_service_adapter
class KafkaProducerAdapter(ServiceAdapter):
    thread_safe = True
    service_name = "kafka_python_producer"

    def do_get_connection(self, *args, **params):
        from kafka import KafkaProducer

        producer = KafkaProducer(*args, **params)
        return producer

    def do_close(self, producer):
        producer.flush()
        producer.close()
